#include "lcd.h"
#include "dht.h"

#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>

#define ADC_CHANNEL 0
#define ADC_AREF_VOLTAGE 5
#define VOLTAGE_MAX 5.5

uint16_t ADC_get()
{
    ADMUX = (ADMUX & 0xF8) | ADC_CHANNEL;

    ADCSRA |= (1 << ADSC);
    while(ADCSRA & (1 << ADSC));

    return ADC;
}

float get_value(uint16_t raw)
{
    return raw * 16 * ADC_AREF_VOLTAGE / 1023;
}

void exemplu_lcd()
{
    LCD_printAt(0x04, "Hello very"); 
    LCD_printAt(0x40, "beautiful World!");
}

void init()
{
    DDRC |= (1 << PC2);
    PORTC |= (1 << PC2);

    ADMUX = (1 << REFS0);
    ADCSRA = (1 << ADEN) | (5 << ADPS0);

    LCD_init();
    exemplu_lcd();
    _delay_ms(5000);
}

int main()
{
    float temperature;
    float old_temperature;
    float humidity;
    float procent;
    uint16_t capacity;
    char temp_buf[16];
    char hum_buf[16];
    char cap_buf1[16];
    char cap_buf2[16];

    init();
    old_temperature = 0.0;
    temperature = 0.0;
    while (1) {
        old_temperature = temperature;
        int rc = dht_gettemperaturehumidity(&temperature, &humidity);
        if (rc == -1) {
            LCD_printAt(0x00, "error reading :(");
            LCD_printAt(0x40, "press RESET ^_^ ");
            break;
        }

        capacity = ADC_get();
        procent = 100 * get_value(capacity) / VOLTAGE_MAX;

        if (procent > 70) {
            DDRD |= (1 << PD1);
            PORTD |= (1 << PD1);
        } else if (procent < 55) {
            PORTD &= ~(1 << PD1);
        }
        sprintf(temp_buf, "Temp:%.2f C    ", temperature);
        LCD_printAt(0x00, temp_buf);
        sprintf(hum_buf, "Humd:%.2f %%    ", humidity);
        LCD_printAt(0x40, hum_buf);
        _delay_ms(2000);
        sprintf(cap_buf1, "Energy level:  ");
        LCD_printAt(0x00, cap_buf1);
        sprintf(cap_buf2, "%.2f -> %.2f  %%", get_value(capacity), procent);
        LCD_printAt(0x40, cap_buf2);
        _delay_ms(2000);
    }
    return 0;
}
